// klpolynomial.cc
//
// Author: allauzen@cs.nyu.edu (Cyril Allauzen)
//
// \file
// Apply a polynomial to a kernel.

#include "kernel/lib/kernel.h"
#include "kernel/lib/composition-kernel.h"

DEFINE_int32(degree, 1, "degree of polynomial");
DEFINE_double(gamma, 1.0F, "gamma of polynomial");
DEFINE_double(coef0, 0.0F, "coef0 of polynomial");

using namespace kernel;

int main(int argc, char** argv) {
  string usage = "Compose kernel with polynomial: ";
  usage += "(gamma * k[x,y] + coef0) ^ degree.\n\n  Usage: ";
  usage += argv[0];
  usage += " in.kar > out.kar\n";

  std::set_new_handler(FailedNewHandler);
  SetFlags(usage.c_str(), &argc, &argv, true);
  if (argc > 3) {
    ShowUsage();
    return 1;
  }
  string in = argc > 1 ? string(argv[1]) : string();
  string out = argc > 2 ? string(argv[2]) : string();

  Kernel< Fst<LogArc> > *kernel = Kernel< Fst<LogArc> >::Read(in);
  if(!kernel)
    LOG(FATAL) << "klpolynomial: failed to read" << in;

  PolynomialKernel< Fst<LogArc> >
    p(kernel, FLAGS_degree, FLAGS_gamma, FLAGS_coef0);
  p.Write(out);
  delete kernel;
  return 0;
}
