// klproduct.cc
//
// Author: allauzen@cs.nyu.edu (Cyril Allauzen)
//
// \file
// Define kernel as product to two kernels.

#include "kernel/lib/kernel.h"
#include "kernel/lib/product-kernel.h"
#include "kernel/lib/rational-kernel.h"

using namespace kernel;

int main(int argc, char** argv) {
  string usage = "Take the product of two kernels:";
  usage += " k1(x,y) * k2(x,y).\n\n  Usage: ";
  usage += argv[0];
  usage += " in1.kar in2.kar > out.kar\n";

  std::set_new_handler(FailedNewHandler);
  SetFlags(usage.c_str(), &argc, &argv, true);
  if (argc < 3 || argc > 4) {
    ShowUsage();
    return 1;
  }
  string in1 = (argc > 1) && strcmp(argv[1], "-") ? string(argv[1]) : string();
  string in2 = (argc > 2) && strcmp(argv[2], "-") ? string(argv[2]) : string();
  string out = argc > 3 ? string(argv[3]) : string();


  // TODO: why not use KernelBase?
  Kernel< Fst<LogArc> > *kernel1 = Kernel< Fst<LogArc> >::Read(in1);
  if(!kernel1)
    LOG(FATAL) << "klproduct: failed to read" << in1;

  Kernel< Fst<LogArc> > *kernel2 = Kernel< Fst<LogArc> >::Read(in2);
  if(!kernel2)
    LOG(FATAL) << "klproduct: failed to read" << in2;

  ProductKernel< Fst<LogArc> > kernel(*kernel1, *kernel2);
  kernel.Write(out);
  delete kernel1;
  delete kernel2;
  return 0;
}
