// klrational.cc
//
// Author: allauzen@cs.nyu.edu (Cyril Allauzen)
//
// \file
// Generate a rational kernel

#include "kernel/lib/kernel.h"
#include "kernel/lib/rational-kernel.h"
#include "kernel/lib/composition-kernel.h"

using namespace kernel;
using namespace fst;

//typedef Fst<LogArc> LogFst;
typedef VectorFst<LogArc> LogVectorFst;

DEFINE_int32(opt, -1,
             "rational function optimization level 0 to 5, -1 for default");
DEFINE_bool(normalize, true,
            "normalize rational kernel: k[x,y] / sqrt(k[x,x] * k[y,y])");
DEFINE_bool(unfactored, false, "input fst is not factored");
DEFINE_bool(asymmetric, false,
            "input fst is not symmetric (when using unfactored fst)");

int main(int argc, char** argv) {
  string usage = "Create a rational kernel.\n\n  Usage: ";
  usage += argv[0];
  usage += " in.fst dataset > rational.kar \n";
  if (argc == 1) {
    cerr << argv[0] << ": use --help for usage" << endl;
    return 1;
  }

  std::set_new_handler(FailedNewHandler);
  SetFlags(usage.c_str(), &argc, &argv, true);
  if (argc < 3 && argc > 4) {
    ShowUsage();
    return 1;
  }
  string out = argc > 3 ? string(argv[3]) : string();

  LogFst *ifst = LogFst::Read(argv[1]);
  if (!ifst)
    LOG(FATAL) << "klrational: cannot open file:" << argv[1];

  LogVectorFst fst(*ifst);
  ArcSort(&fst, ILabelCompare<LogArc>());

  DataSet<LogFst> *data = new DataSet<LogFst>(argv[2]);

  if (FLAGS_unfactored) {
    UnfactoredRationalKernel<LogArc>
      kernel(fst, FLAGS_opt, FLAGS_normalize, !FLAGS_asymmetric,
             FLAGS_asymmetric ? kCacheNorm : kCacheValue);
    kernel.SetData(data);
    kernel.Write(out);
  } else {
    RationalKernel<LogArc> kernel(fst, FLAGS_opt, FLAGS_normalize);
    kernel.SetData(data);
    kernel.Write(out);
  }

  delete ifst;
  return 0;
}
