
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// Author: rostami@google.com (Afshin Rostamizadeh)

#ifndef KERNEL_LIB_FEATURE_WEIGHTED_KERNEL_H_
#define KERNEL_LIB_FEATURE_WEIGHTED_KERNEL_H_

#include "kernel/lib/kernel.h"
#include "kernel/lib/basic-kernel.h"
#include "kernel/lib/feature-weighted-function.h"

namespace kernel {

typedef DenseVector<double> DenseVectorD;

template <class V>
class FeatureWeightedKernel :
  public BasicKernel< FeatureWeightedFunction<V>,
                      BasicCache<V,V> > {
 public:
  typedef FeatureWeightedFunction<V> Function;
  typedef typename Function::Object Object;
 private:
  typedef BasicCache<V,V> Cache;

 public:
  FeatureWeightedKernel(const DenseVector<typename V::ElementType> &weights,
                        bool normalize = true,
                        uint32 cache_type = kCacheFeature | kCacheValue)
      : BasicKernel<Function, Cache>(new Function(weights),
                                     new Cache(cache_type),
                                     normalize) {
        BasicKernel<Function, Cache>::Impl()->SetType("feature_weighted");
      }

  FeatureWeightedKernel(const BasicKernel<Function, Cache> &kernel)
      : BasicKernel<Function, Cache>(kernel) {
        BasicKernel<Function, Cache>::Impl()->SetType("feature_weighted");
      }

  static FeatureWeightedKernel<V> *Read(istream &strm,
                                        const KernelReadOptions &opts) {
    KernelReadOptions ropts(opts);
    ropts.type = "feature_weighted";
    BasicKernelImpl<Function, Cache>* impl =
        BasicKernelImpl<Function, Cache>::Read(strm, ropts);
    return impl ? new FeatureWeightedKernel<V>(impl) : 0;
  }

  static FeatureWeightedKernel<V> *Read(const string &filename) {
    if (!filename.empty()) {
      ifstream strm(filename.c_str());
      if (!strm) {
        LOG(ERROR) << "RationalKernel::Read: Can't open file: " << filename;
        return 0;
      }
      return Read(strm, KernelReadOptions(filename));
    } else {
      return Read(std::cin, KernelReadOptions("standard input"));
    }
  }

 private:
  explicit FeatureWeightedKernel(BasicKernelImpl<Function, Cache> *impl) :
      BasicKernel<Function, Cache>(impl) {
    BasicKernel<Function, Cache>::Impl()->SetType("feature_weighted");
  }
};

} // namespace kernel

#endif  // KERNEL_LIB_FEATURE_WEIGHTED_KERNEL_H_
