// kernel-function.h
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// Author: allauzen@cs.nyu.edu (Cyril Allauzen)
//
// \file
// Abstract base class for kernel function representation.

#ifndef KERNEL_LIB_KERNEL_FUNCTION_H__
#define KERNEL_LIB_KERNEL_FUNCTION_H__

#include <string>

#include "kernel/lib/compat.h"


namespace kernel {


template <class X>
class Function {
 public:
  typedef X Object;

  virtual ~Function() {}

  virtual double operator()(const X&, const X&) const = 0;
  virtual const string Type() const = 0;
  virtual const uint64 Properties() const = 0;
};


template <class X, class F>
class FeaturedFunction : public Function<X> {
 public:
  typedef X Object;
  typedef F Feature;

  virtual ~FeaturedFunction() {}


  virtual F* GetFeature(const X&) const = 0;
  virtual double Product(const F&, const F&) const = 0;
};

}


#endif  // KERNEL_LIB_KERNEL_FUNCTION_H__
