// properties.h
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//
// Author: allauzen@cs.nyu.edu (Cyril Allauzen)
//
// \file
// Definitions of property bits.

#ifndef KERNEL_LIB_PROPERTIES_H__
#define KERNEL_LIB_PROPERTIES_H__

#include "kernel/lib/compat.h"

namespace kernel {

const uint64 kFeatured =           0x0000000000000001ULL;

const uint64 kNormalized =         0x0000000000010000ULL;
const uint64 kUnnormalized  =      0x0000000000020000ULL;

const uint64 kSymmetric =          0x0000000000040000ULL;
const uint64 kAsymmetric =         0x0000000000080000ULL;

const uint64 kPosDefinite =        0x0000000000100000ULL;
const uint64 kNegDefinite =        0x0000000000200000ULL;
const uint64 kNotDefinite =        0x0000000000400000ULL;

//
// Composite Properties
//

// PDS: Positive Definite Symmetric
const uint64 kPDS = kSymmetric | kPosDefinite;

// NDS: Negative Definite Symmetric
const uint64 kNDS = kSymmetric | kNegDefinite;


const uint64 kKernelProperties = kFeatured | kSymmetric | kAsymmetric |
                                 kPosDefinite | kNegDefinite | kNotDefinite |
                                 kNormalized | kUnnormalized;


//
// Properties for Kernel operations
//

uint64 ExternalProductProperties(uint64 inprops, double a);
uint64 ProductProperties(uint64 inprops1, uint64 inprops2);
uint64 SumProperties(uint64 inprops1, uint64 inprops2, double a1, double a2);

}  // namespace kernel


#endif  // KERNEL_LIB_PROPERTIES_H__
